/*
	Pixel Scale - A Screen Ruler and Planimeter
	Copyright © 2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Pixel Scale - version 2.2
	7 August, 2007
	Copyright © 2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

var deltaX = 0;
var downX  = 0;

var downCalibrationH = 0;
var downCalibrationV = 0;
var downRotation = 0;
var downWidth = 0;

var gCalibrationGain = 1.0;
var gRotationGain = 1.0;
var gWidthGain    = 4.0;

var minW = 80;
var maxW = 1536;

var resizing = false;
var calibrating = false;
var clockwise = 1;	// anti-clockwise = -1

main_window.width  = 2*screen.width;
main_window.height = 2*screen.height;

var ghOffset = screen.availLeft + screen.availWidth - 200;

ruler.hOffset = legend.hOffset = label.hOffset = ghOffset + 100;
button.hOffset = ghOffset + 50;
buttonText.hOffset = ghOffset + 50;

var gvOffset = screen.availTop + screen.availHeight - ruler.height;

ruler.vOffset = legend.vOffset = label.vOffset = gvOffset;
button.vOffset = gvOffset + 20;
buttonText.vOffset = gvOffset + 25;

var screenH = 17.025;	// in inches
var screenV = 10.67;	// in inches

var ppiH = screen.width  / screenH;
var ppiV = screen.height / screenV;

var scaleFactor = 100;	// 100 for inches, 101.6 for centimeters

var pixelMode = 100;

function normalized(rotation)
{
	rotation = ((rotation % 360) + 360) % 360;
	if (rotation > 180) { rotation = rotation - 360; }
	return rotation;
}

function thePPI(theta, h, v, pixelMode)	// theta in degrees
{
	if (pixelMode !== 0) { return pixelMode; }
	
	var a = h * Math.sin(theta * Math.PI / 180);
	var b = v * Math.cos(theta * Math.PI / 180);
	return h * v / Math.sqrt( a*a + b*b);
}

function updatePrefs()
{
	screenH = screen.width / screen.resolution;	// estimate screen width
	var screenHPref = parseFloat(preferences.screenHPref.value, 10);
	if (isNaN(screenHPref)) { screenHPref = screenH; }
	if ((0.5*screenH <= screenHPref) && (screenHPref <= 2*screenH))
	{
		screenH = screenHPref;
	}
	preferences.screenHPref.value = String(screenHPref);
	ppiH = screen.width  / screenH;
	var hAdjustPref = Number(preferences.hAdjustPref.value);
	ppiH = (1 + 0.0001*hAdjustPref) * ppiH;

	screenV = screen.height / screen.resolution;	// estimate screen height
	var screenVPref = parseFloat(preferences.screenVPref.value, 10);
	if (isNaN(screenVPref)) { screenVPref = screenV; }
	if ((0.5*screenV <= screenVPref) && (screenVPref <= 2*screenV))
	{
		screenV = screenVPref;
	}
	preferences.screenVPref.value = String(screenVPref);
	ppiV = screen.height  / screenV;
	var vAdjustPref = Number(preferences.vAdjustPref.value);
	ppiV = (1 + 0.0001*vAdjustPref) * ppiV;

	setMode(preferences.modePref.value);
	updateArea();

	ruler.width  = legend.width = maxW;
	setWidth(preferences.widthPref.value);
	if (preferences.clockwisePref.value == "1") { clockwise = 1; } else { clockwise = -1; }

	var rotation = parseInt(preferences.rotationPref.value, 10);
	if (isNaN(rotation)) { rotation = 0; }
	rotation = normalized(rotation);
	buttonText.data  = preferences.rotationPref.value = rotation;	// as set by user
	buildVitality(buttonText.data);
	
	ruler.rotation = legend.rotation = label.rotation = clockwise * rotation;

	ruler.width = legend.width = ruler.srcWidth * thePPI(ruler.rotation, ppiH, ppiV, pixelMode) / scaleFactor;

	ruler.colorize   = preferences.colorPref.value;
	label.colorize   = legend.colorize  = preferences.legendColorPref.value;
	button.colorize  = preferences.buttonColorPref.value;
	buttonText.color = preferences.textColorPref.value;
}

function mouseDn(img)
{
	downX = system.event.hOffset + deltaX;
	downCalibrationH = Number(preferences.hAdjustPref.value);
	downCalibrationV = Number(preferences.vAdjustPref.value);
	downRotation = ruler.rotation;
	downWidth = Number(preferences.widthPref.value);
	var sem = system.event.modifiers;
	if (optionKeyDown(sem) && shiftKeyDown(sem))
	{
		ruler.rotation = ((ruler.rotation % 360) + 360) % 360;
		if (((ruler.rotation % 90) === 0) && (pixelMode === 0)) { calibrating = true; }
		else { beep(); alert('Calibration can only be done when the ruler is horizontal or vertical and is in the 1 centimetre or 1 inch mode.'); }
	}
	else if (optionKeyDown(sem))
	{
		ruler.rotation = legend.rotation = label.rotation = 0; resizing = true; buttonText.data = 0; updateNow();
	}
	else if (shiftKeyDown(sem))
	{
		if (ruler.rotation % 90 === 0) { ruler.rotation = legend.rotation = label.rotation = (ruler.rotation + 90) % 360; }
		else { ruler.rotation = legend.rotation = label.rotation = 0; }
		buttonText.data = preferences.rotationPref.value = normalized(clockwise * ruler.rotation);
	}
}

function mouseUp(img)
{
	if (resizing) { ruler.rotation = legend.rotation = label.rotation = downRotation; resizing = false; buttonText.data = normalized(clockwise * downRotation); }
	else if (calibrating) { calibrating = false; }
	buildVitality(buttonText.data);
}
		
function mouseMv(img)
{
	var mvX = system.event.hOffset + deltaX;
	if (resizing) { reSize(mvX - downX); }
	else if (calibrating) { calibrate(mvX - downX); }
	else { rotate(mvX - downX); }
}

function calibrate(cDeltaX)
{
	var gain = gCalibrationGain;
	if (capslockKeyDown(system.event.modifiers)) { gain = 0.5*gain; }
	
	if ((ruler.rotation % 180) === 0)	// horizontal calibration
	{
		var hAdjustPref = downCalibrationH + gain*cDeltaX;
		if (hAdjustPref < -100) { hAdjustPref = -100; }
		else if (hAdjustPref > 100) { hAdjustPref = 100; }
		preferences.hAdjustPref.value = String(hAdjustPref);
		ppiH = (1 + 0.0001*hAdjustPref) * screen.width  / screenH;
	}
	else	// vertical calibration
	{
		var vAdjustPref = downCalibrationV + gain*cDeltaX;
		if (vAdjustPref < -100) { vAdjustPref = -100; }
		else if (vAdjustPref > 100) { vAdjustPref = 100; }
		preferences.vAdjustPref.value = String(vAdjustPref);
		ppiV = (1 + 0.0001*vAdjustPref) * screen.height  / screenV;
	}
	ruler.width  = legend.width = ruler.srcWidth * thePPI(ruler.rotation, ppiH, ppiV, pixelMode) / scaleFactor;
	updateArea();
}

function rotate(rDeltaX)
{
	var gain = gRotationGain;
	if (capslockKeyDown(system.event.modifiers)) { gain = 0.5*gain; }
	ruler.rotation = legend.rotation = label.rotation = Math.round(360 + downRotation + gain*rDeltaX) % 360;
	ruler.width  = legend.width = ruler.srcWidth * thePPI(ruler.rotation, ppiH, ppiV, pixelMode) / scaleFactor;
	buttonText.data = preferences.rotationPref.value = normalized(clockwise * ruler.rotation);
}

function reSize(wDeltaX)
{
	var gain = gWidthGain;
	if (capslockKeyDown(system.event.modifiers)) { gain = 0.5*gain; }
	var width = Math.round(downWidth + gain*wDeltaX);
	setWidth(width);
}

function setWidth(width)
{
	if (width < minW) { width = minW; }
	if (width > maxW) { width = maxW; }
	width = minW * Math.ceil(width/minW) * thePPI(ruler.rotation, ppiH, ppiV, pixelMode) / scaleFactor;
	ruler.clipRect   = "0, 0, " + width + ", 36";
	legend.clipRect  = "0, 0, " + width + ", 20";
	preferences.widthPref.value = String(width);
}
