/*
	Modifier-key Inspection Functions
	Copyright © 2006-2007 Harry Whitfield

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation; either version 2 of the License, or (at your
	option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	Modifier-key Inspection Functions 1.0
	4 February, 2007
	Copyright © 2006-2007 Harry Whitfield
	mailto:g6auc@arrl.net
*/

// Modifier-key Inspection Functions
// 
// Macintosh: capslock+shift+control+option+numlock+fkey
// Windows:   capslock+shift+control+alt
//
// Numlock is ignored in some circumstances.
// Option is regarded as equivalent to Alt
// There are some functions which treat fkey as equivalent to capslock

function modKeysDown(sem) // returns number of modifier keys that are down, ignoring numlockKey
{
    if (sem === "") { return 0; }
    var num = sem.split('+').length;
    if (sem.indexOf('numlock') >= 0) { num -= 1; } // On iBook G4, fnKey produces numlock+fkey
    return num;
}

function keysDown(sem,num)         { return  (modKeysDown(sem) == num); }

function noKeysDown(sem)           { return  (sem === ''           ); }

function capslockKeyOnlyDown(sem)  { return  (sem == 'capslock'    ); }
function shiftKeyOnlyDown(sem)     { return  (sem == 'shift'       ); }
function controlKeyOnlyDown(sem)   { return  (sem == 'control'     ); }
function optionKeyOnlyDown(sem)    { return  (sem == 'option'      ) ||
											 (sem == 'alt'         ); }
function numlockKeyOnlyDown(sem)   { return  (sem == 'numlock'     ); }
function fnKeyOnlyDown(sem)        { return  (sem == 'fkey'        ) ||
											 (sem == 'numlock+fkey'); }

function capslockKeyDown(sem)      { return  (sem.indexOf('capslock') >= 0); }
function shiftKeyDown(sem)         { return  (sem.indexOf('shift'   ) >= 0); }
function controlKeyDown(sem)       { return  (sem.indexOf('control' ) >= 0); }
function optionKeyDown(sem)        { return ((sem.indexOf('option'  ) >= 0) ||
											 (sem.indexOf('alt'     ) >= 0));}
function numlockKeyDown(sem)       { return  (sem.indexOf('numlock' ) >= 0); }
function fnKeyDown(sem)            { return  (sem.indexOf('fkey'    ) >= 0); }
function fnOrCapslockKeyDown(sem)  { return ((sem.indexOf('fkey'    ) >= 0) ||
											 (sem.indexOf('capslock') >= 0));}

function capslockKeyUp(sem)        { return  (sem.indexOf('capslock')  < 0); }
function shiftKeyUp(sem)           { return  (sem.indexOf('shift'   )  < 0); }
function controlKeyUp(sem)         { return  (sem.indexOf('control' )  < 0); }
function optionKeyUp(sem)          { return ((sem.indexOf('option'  )  < 0) &&
											 (sem.indexOf('alt'     )  < 0));}
function numlockKeyUp(sem)         { return  (sem.indexOf('numlock' )  < 0); }
function fnKeyUp(sem)              { return  (sem.indexOf('fkey'    )  < 0); }
function fnOrCapslockKeyUp(sem)    { return ((sem.indexOf('fkey'    )  < 0) &&
											 (sem.indexOf('capslock')  < 0));}
